﻿// Alternative JSON parser
// Why: because JSON.parse(data) has a memory leak and we call it at over 20Hz !!!
//
// piWeabCAT fires repetitive ajax calls at the RPi server
// Web broswer javascript was leaking memory at a rate of over 1 Mbyte per minute.
// -- piWebCAT eventually slows to an unacceptable pace.
// This was only realised very late in development because:
// 1.  I noted that Hamlib was returning data from the server 3.5x faster than my ASCII, YAESU5 and CIV modes.
//     I fixed this - which increased the data rate and thereby increase the rate of memory leak.
// 2.  Also, the piWebCAT development process is continually modify-test-modify test
//     and so it was unusual to run for long periods.
// 
// I was getting ready to write my own parser when I found fakeParseJSON()
//
//   https://lihautan.com/json-parser-with-javascript/
//
// Many thanks to Tan Li Hau for this !!!
// 
// I was still getting an unacceptable memory leak rate, despite trying different 
// ways of achieving catAnswerArray = fakeParseJSON(data)
//
// I then realised that all the repetitive aj read and write returned the same six - item array
//
// I defined the array as a global with default values sies to encompasss the  sie of returned data
// and modified fakseParseJSON   to catParseJSON(data, caa)
//
// If caa is true then the JSON is parse directly in to global array
//
// I caa is false then it behaves as previously .... used at startup - no repetition so small lea ok.

"use strict";




var catAnswerArray = 
	{"param":  "ABCD",
	 "value":  1.000,
	 "nchar":  22,
	 "code":   "ABCD",
	 "rxab":   'A',
	 "error":  RPI_ERROR_NO_ERROR,
	 "ts":		 199999
  };



function catParseJSON(str, caa)    // caa = catAnswerArray
{
  let i = 0;
  const value = parseValue();
  expectEndOfInput();
  return value;


  function parseObject() 
  {
    if (str[i] === "{") 
    {
      i++;
      skipWhitespace();
      
      const result = {};
      
      let initial = true;
      // if it is not '}',
      // we take the path of string -> whitespace -> ':' -> value -> ...
      while (i < str.length && str[i] !== "}") {
        if (!initial) {
          eatComma();
          skipWhitespace();
        }
        const key = parseString();
        if (key === undefined) {
          expectObjectKey();
        }
        skipWhitespace();
        eatColon();
        const value = parseValue();
        
        if(caa)
        {
        	switch(key)
        	{
        		case "param":   catAnswerArray["param"] = value; break;
        		case "value":   catAnswerArray["value"] = value; break;
        		case "nchar":   catAnswerArray["nchar"] = value; break;
        		case "code":    catAnswerArray["code"]  = value; break;
        		case "rxab":    catAnswerArray["rxab"]  = value; break;
        		case "error":   catAnswerArray["error"] = value; break;
        		case "ts":      catAnswerArray["ts"] = value; break;
        	}
        }
        else
        {	
        	result[key] = value;
        }
        initial = false;
      }
      expectNotEndOfInput("}");
      // move to the next character of '}'
      i++;
      if(caa){return;} else {return result;}
    }
  }

  function parseArray()
  {
    if (str[i] === "[") 
    {
      i++;
      skipWhitespace();

      const result = [];
      let initial = true;
      while (i < str.length && str[i] !== "]") 
      {
        if (!initial) 
        {
          eatComma();
        }
        const value = parseValue();
        result.push(value);
        initial = false;
      }
      expectNotEndOfInput("]");
      // move to the next character of ']'
      i++;
      return result;
    }
  }

  function parseValue() 
  {
    skipWhitespace();
    const value =
      parseString() ??
      parseNumber() ??
      parseObject() ??
      parseArray() ??
      parseKeyword("true", true) ??
      parseKeyword("false", false) ??
      parseKeyword("null", null);
    skipWhitespace();
    return value;
  }

  function parseKeyword(name, value) 
  {
    if (str.slice(i, i + name.length) === name) {
      i += name.length;
      return value;
    }
  }

  function skipWhitespace() 
  {
    while (str[i] === " " || str[i] === "\n" || str[i] === "\t" || str[i] === "\r") 
    {
      i++;
    }
  }



  function parseString() 
  {
    var ctlChar;
    var char = 'x';
    if (str[i] === '"') 
    {
      i++;
      let result = "";
      while (i < str.length && str[i] !== '"') 
      {						
				if (str[i] === "\\") 
	      {
          char = str[i + 1];
         	ctlChar = false;
         	if(char === '"') {ctlChar = true;}
          if(char === "\\") {ctlChar = true; char = "\\";}
          if(char === "/") {ctlChar = true; char = "\/";}
          if(char === "b") {ctlChar = true; char = "\b";}
          if(char === "f") {ctlChar = true; char = "\f";}
          if(char === "n") {ctlChar = true; char = "\n";}
          if(char === "r") {ctlChar = true; char = "\r";}
          if(char === "t") {ctlChar = true;}
          					
					if(ctlChar)
	        {
            result += char;
            i++;
          } 
          else if (char === "u") 
          {
            if (
              isHexadecimal(str[i + 2]) &&
              isHexadecimal(str[i + 3]) &&
              isHexadecimal(str[i + 4]) &&
              isHexadecimal(str[i + 5])
               ) 
            {
              result += String.fromCharCode(parseInt(str.slice(i + 2, i + 6), 16));
              i += 5;
            } 
            else 
            {
              i += 2;
              expectEscapeUnicode(result);
            }
          } 
          else 
          {
            expectEscapeCharacter(result);
          }
        } 
        else 
        {
          result += str[i];
        }
        i++; 
      }
      expectNotEndOfInput('"');
      i++;
      return result;
    }
  }

  function isHexadecimal(char) {
    return (
      (char >= "0" && char <= "9") ||
      (char.toLowerCase() >= "a" && char.toLowerCase() <= "f")
    );
  }

  function parseNumber() 
  {
    let start = i;
    if (str[i] === "-") 
    {
      i++;
      expectDigit(str.slice(start, i));
    }
    if (str[i] === "0") {i++;} 
    else if (str[i] >= "1" && str[i] <= "9") 
    {
      i++;
      while (str[i] >= "0" && str[i] <= "9") {i++;}
    }

    if (str[i] === ".") 
    {
      i++;
      expectDigit(str.slice(start, i));
      while (str[i] >= "0" && str[i] <= "9") {i++;}
    }
    if (str[i] === "e" || str[i] === "E") {
      i++;
      if (str[i] === "-" || str[i] === "+") {
        i++;
      }
      expectDigit(str.slice(start, i));
      while (str[i] >= "0" && str[i] <= "9") {
        i++;
      }
    }
    if (i > start) {
      return Number(str.slice(start, i));
    }
  }

  function eatComma() 
  {
    expectCharacter(",");
    i++;
  }

  function eatColon() 
  {
    expectCharacter(":");
    i++;
  }

  // error handling
  function expectNotEndOfInput(expected) 
  {
    if (i === str.length) 
    {
      printCodeSnippet(`Expecting a \`${expected}\` here`);
      throw new Error("JSON_ERROR_0001 Unexpected End of Input");
    }
  }

  function expectEndOfInput() 
  {
    if (i < str.length) 
    {
      printCodeSnippet("Expecting to end here");
      throw new Error("JSON_ERROR_0002 Expected End of Input");
    }
  }

  function expectObjectKey() 
  {
    printCodeSnippet(`Expecting object key here

For example:
{ "foo": "bar" }
  ^^^^^`);
    throw new Error("JSON_ERROR_0003 Expecting JSON Key");
  }

  function expectCharacter(expected) 
  {
    if (str[i] !== expected) 
    {
      printCodeSnippet(`Expecting a \`${expected}\` here`);
      throw new Error("JSON_ERROR_0004 Unexpected token");
    }
  }

  function expectDigit(numSoFar) 
  {
    if (!(str[i] >= "0" && str[i] <= "9")) 
    {
      printCodeSnippet(`JSON_ERROR_0005 Expecting a digit here

For example:
${numSoFar}5
${" ".repeat(numSoFar.length)}^`);
      throw new Error("JSON_ERROR_0006 Expecting a digit");
    }
  }

  function expectEscapeCharacter(strSoFar) 
  {
    printCodeSnippet(`JSON_ERROR_0007 Expecting escape character

For example:
"${strSoFar}\\n"
${" ".repeat(strSoFar.length + 1)}^^
List of escape characters are: \\", \\\\, \\/, \\b, \\f, \\n, \\r, \\t, \\u`);
    throw new Error("JSON_ERROR_0008 Expecting an escape character");
  }

  function expectEscapeUnicode(strSoFar) 
  {
    printCodeSnippet(`Expect escape unicode

For example:
"${strSoFar}\\u0123
${" ".repeat(strSoFar.length + 1)}^^^^^^`);
    throw new Error("JSON_ERROR_0009 Expecting an escape unicode");
  }

  function printCodeSnippet(message) 
  {
    const from = Math.max(0, i - 10);
    const trimmed = from > 0;
    const padding = (trimmed ? 4 : 0) + (i - from);
    const snippet = [
      (trimmed ? "... " : "") + str.slice(from, i + 1),
      " ".repeat(padding) + "^",
      " ".repeat(padding) + message
    ].join("\n");
    console.log(snippet);
  }
}


// console.log("Try uncommenting the fail cases and see their error message");
// console.log("↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓");

// Fail cases:
printFailCase("-");
printFailCase("-1.");
printFailCase("1e");
printFailCase("-1e-2.2");
printFailCase("{");
printFailCase("{}{");
printFailCase('{"a"');
printFailCase('{"a": "b",');
printFailCase('{"a":"b""c"');
printFailCase('{"a":"foo\\}');
printFailCase('{"a":"foo\\u"}');
printFailCase("[");
printFailCase("[][");
printFailCase("[[]");
printFailCase('["]');

function printFailCase(json) 
{
  try 
  {
		console.log(`catParseJSON('${json}', false)`);
		catParseJSON(json, false);
 	} 
 	catch (error) 
 	{
    console.error(error);
 	}
}
